# Based on IDS Imaging Development Systems GmbH's PyUeye for camera operation

#Libraries
from pyueye import ueye
import numpy as np
import cv2
#from matplotlib import pyplot as plt
#import sys
#from PIL import Image
import time

#---------------------------------------------------------------------------------------------------------------------------------------

#Variables
hCam = ueye.HIDS(0)             #0: first available camera;  1-254: The camera with the specified camera ID
sInfo = ueye.SENSORINFO()
cInfo = ueye.CAMINFO()
pcImageMemory = ueye.c_mem_p()
MemID = ueye.int()
rectAOI = ueye.IS_RECT()
pitch = ueye.INT()
nBitsPerPixel = ueye.INT(24)    #24: bits per pixel for color mode; take 8 bits per pixel for monochrome
channels = 3                    #3: channels for color mode(RGB); take 1 channel for monochrome
m_nColorMode = ueye.INT()		# Y8/RGB16/RGB24/REG32
bytes_per_pixel = int(nBitsPerPixel / 8)

#---------------------------------------------------------------------------------------------------------------------------------------

# Starts the driver and establishes the connection to the camera
nRet = ueye.is_InitCamera(hCam, None)
# Reads out the data hard-coded in the non-volatile camera memory and writes it to the data structure that cInfo points to
nRet = ueye.is_GetCameraInfo(hCam, cInfo)
# You can query additional information about the sensor type used in the camera
nRet = ueye.is_GetSensorInfo(hCam, sInfo)
nRet = ueye.is_ResetToDefault( hCam)
# Set display mode to DIB
nRet = ueye.is_SetDisplayMode(hCam, ueye.IS_SET_DM_DIB)

m_nColorMode = ueye.IS_CM_MONO8
nBitsPerPixel = ueye.INT(8)
bytes_per_pixel = int(nBitsPerPixel / 8)

# Can be used to set the size and position of an "area of interest"(AOI) within an image
nRet = ueye.is_AOI(hCam, ueye.IS_AOI_IMAGE_GET_AOI, rectAOI, ueye.sizeof(rectAOI))

width = rectAOI.s32Width
height = rectAOI.s32Height

#---------------------------------------------------------------------------------------------------------------------------------------

# Allocates an image memory for an image having its dimensions defined by width and height and its color depth defined by nBitsPerPixel
nRet = ueye.is_AllocImageMem(hCam, width, height, nBitsPerPixel, pcImageMemory, MemID)
nRet = ueye.is_SetImageMem(hCam, pcImageMemory, MemID)
nRet = ueye.is_SetColorMode(hCam, m_nColorMode)

# Activates the camera's live video mode (free run mode)
nRet = ueye.is_CaptureVideo(hCam, ueye.IS_DONT_WAIT)

# Enables the queue mode for existing image memory sequences
nRet = ueye.is_InquireImageMem(hCam, pcImageMemory, MemID, width, height, nBitsPerPixel, pitch)
print("Press q to leave the programm")

#---------------------------------------------------------------------------------------------------------------------------------------

#Nucleo Integration part
from serial import Serial
import serial.tools.list_ports

ports = serial.tools.list_ports.comports()
# To obtain the list of the communication ports
for port, desc, hwid in sorted(ports):
    print("{}: {}".format(port, desc))
# To select the port to use
selectPort = input("Select a COM port : ")    
print(f"Port Selected : COM{selectPort}")
# To open the serial communication at a specific baudrate
serNuc = Serial('COM'+str(selectPort), 115200)  # Under Windows only

LastWrite = time.time()

#---------------------------------------------------------------------------------------------------------------------------------------

# Continuous image display
while(nRet == ueye.IS_SUCCESS):

    #extract the data of image memory
    array = ueye.get_data(pcImageMemory, width, height, nBitsPerPixel, pitch, copy=False)
    bytes_per_pixel = int(nBitsPerPixel / 8)

    # ...reshape it in an numpy array...
    frame = np.reshape(array,(height.value, width.value, bytes_per_pixel))

#---------------------------------------------------------------------------------------------------------------------------------------
    #Detection Area
    
    RectX = 200
    RectY=30
    RectWidth = 240
    RectHeight = 400

    #DATA PROCESSING
    
    #constrasting:
    frame = cv2.convertScaleAbs(frame, 1.5, 5)

    #blurring :
    blurred = cv2.GaussianBlur(frame, (5, 5), 10) #4eme argument contrôle l'intensité du blurring

    # #setting the threshold of the gray image :
    thresh = cv2.threshold(blurred, 150, 255, cv2.THRESH_BINARY)[1] #2emme argument contrôle le seuillage, 3emme la couleur de l'image seuillée, type de seuillage

    #Find the contours
    contours, _ = cv2.findContours(thresh, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

    for contour in contours:
        
        # compute the bounding box for the contour
        (x, y, w, h) = cv2.boundingRect(contour)
    
        # reject contours outside size range
        if w < 30 or h < 30 :
              continue
        # make sure the box is inside the frame
        if x <= RectX or y <= RectY or x+w >= RectX+RectWidth or y+h >= RectY+RectHeight:
          continue

        # cv2.approxPloyDP() function to approximate the shape
        approx = cv2.approxPolyDP(contour, 0.018 * cv2.arcLength(contour, True), True)

        # using drawContours() function
        cv2.drawContours(thresh, [contour], 0, (180, 0, 0), 3)

        # finding center point of shape
        M = cv2.moments(contour)
        x = int(M['m10'] / M['m00'])
        y = int(M['m01'] / M['m00'])

        # putting shape name at center of each shape
        shapeName = ''
        if len(approx) == 3:
            shapeName = 'Triangle'

        elif len(approx) == 4:
            shapeName = 'Square'

        elif len(approx) == 5:
            shapeName = 'Pentagon'

        elif len(approx) == 6:
            shapeName = 'Hexagon'

        elif len(approx) == 12:
            shapeName = 'Etoile'
        else:
            shapeName = 'Circle'
        
        if shapeName != '':
            cv2.putText(thresh, shapeName, (x, y), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (180, 0, 0), 2)
            NewWrite = time.time()
            #send to Nucleo only every 1 second
            
            if NewWrite - LastWrite > 1:
                LastWrite = time.time()
                print('sending data: ', shapeName)
                serNuc.write(bytes(shapeName[0], 'ascii'))
                # while serNuc.inWaiting() == 0:
                #     pass
                # data_rec = serNuc.read(1)  # bytes
                # print(str(data_rec))

#---------------------------------------------------------------------------------------------------------------------------------------

    #DISPLAY
    
    RectEnd = (RectX+RectWidth,RectY+RectHeight)
    cv2.rectangle(thresh, (RectX,RectY), RectEnd, (255,0,0),2)
    
    cv2.imshow('shapes', thresh)


    # Press q if you want to end the loop
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break
#---------------------------------------------------------------------------------------------------------------------------------------

# Releases an image memory that was allocated using is_AllocImageMem() and removes it from the driver management
ueye.is_FreeImageMem(hCam, pcImageMemory, MemID)

# Disables the hCam camera handle and releases the data structures and memory areas taken up by the uEye camera
ueye.is_ExitCamera(hCam)

# Destroys the OpenCv windows
cv2.destroyAllWindows()

